#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
import base64
import hashlib 
import time
import string
from django.conf import settings
from npai_conf.common  import fileUtils
from django.utils.translation import ugettext as _
import traceback

from state.get_state import GetCommand

class NpaiSetting():
    def __init__(self):
        self.logXml = '/opt/nsfocus/NPAI/data/status/links.xml'
        self.cmdXml = '/opt/nsfocus/NPAI/data/status/cmdstatus.xml'
        self.localIpConfig = '/opt/nsfocus/NPAI/config/LOCALADDR'
        self.clientXml = '/opt/nsfocus/NPAI/config/client.xml'
        self.timeout = '/opt/nsfocus/NPAI/config/TIMEOUT'
        self.recoverXml = '/opt/nsfocus/NPAI/config/recover.xml'
        self.npaiVersion = '/opt/nsfocus/NPAI/config/version.xml'
        self.autoUpdatePath = '/opt/nsfocus/NPAI/config/AUTOUPDATE'
        self.editesppPath = '/opt/nsfocus/NPAI/config/EDITESPP'
        self.esppAddress = 'espp.api.nsfocus.com'
            
    def getNpaiSettingInfo(self):
        #os.system(cmd)
        pass

    def getLocalIp(self):
        data = fileUtils.getFileContent(self.localIpConfig)
        if data is None:
            return ''
        else:
            return data.strip()
        
    def getCilentList(self):
        _xmlUtils = fileUtils.XmlUtils(self.clientXml)
        brokerNodes = _xmlUtils.getAllChildren('broker')
        
        brokeNodeList = []
        
        def formatValue(key,value):
            if value is None:
                value =''
            #base64 encode
            if key in ['passwd','ftp_pwd','pkey']:
                value = base64.b64decode(value)
                
                if (value.strip()!=''):
                    h=hashlib.md5()
                    h.update(value)
                    value = h.hexdigest()[0:8]
            
            return value
                
        for brokerNode in brokerNodes:
            '''
            if brokerNode.get('id')=='0':
                continue
            '''
            if not brokerNode.get('id') in ['0', '1']:
                continue
            
            nodeDict = _xmlUtils.parseXml2Dict(brokerNode,formatValue)
            brokeNodeList.append(nodeDict)
    
        brokeNodeList =sorted(brokeNodeList,key = lambda x:x['id'])     
        return brokeNodeList

    def updateNpaiSetting(self,dict):	
        self.updateLocalIp(dict.get("localIp"))
        
        self.updateClientXml(dict)
        
    def updateLocalIp(self,data):
        #os.system(cmd)
        fileUtils.writeFile(data,self.localIpConfig)
    
    def updateClientXml(self,dict):
        def formatValue(key,oldValue,dictValue):
                #base64 encode
                if key in ['passwd','ftp_pwd','pkey']:
                    if dictValue is None:
                        dictValue = ''
                        
                    if oldValue.strip()!='' :
                        oldValue = base64.b64decode(oldValue)
                        h=hashlib.md5()
                        h.update(oldValue)
                        value = h.hexdigest()[0:8]
                        if(value!=dictValue):
                            return base64.b64encode(dictValue)
                        else: 
                            return None
                    return base64.b64encode(dictValue)
                else:
                    return dictValue
                
                return None
        
        def formatAttrValue(key,dictValue):
                if ((key=='inuse' or key=='uploadlog') and dictValue is None):
                    return 'no'
                return None
            
        def formatEsppValue(key,oldValue,dictValue):
             if key == 'host' and dictValue is not None:
                 return self.esppAddress
             return formatValue(key,oldValue,dictValue)
                
        _xmlUtils = fileUtils.XmlUtils(self.clientXml)
        brokerNodes = _xmlUtils.getAllChildren('broker')
       
        for brokerNode in brokerNodes:
            brokerId = brokerNode.get('id')
            
            dictKeySuffix = '_' + brokerId
            
            if brokerId=='0':
                _xmlUtils.updateTree(brokerNode,dict,dictKeySuffix,formatAttrValue,formatEsppValue)
            else:
                if brokerId=='1':
                    _xmlUtils.updateTree(brokerNode,dict,dictKeySuffix,formatAttrValue,formatValue)
            
        _xmlUtils.writeXml()

    def rebootNpai(self):
        commander = GetCommand()
        ret = commander.sync_command(28, 0, 'restart')

        return ret[0]
    
    '''npai version xml:
       <?xml version="1.0" encoding="UTF-8"?>
        <root>
        <version>1.1.2.57109</version>
        <updatetime>1403854001</updatetime>
        </root>
       npai auto update config:
       true
    '''
    def getNpaiInfo(self):
        _xmlUtils = fileUtils.XmlUtils(self.npaiVersion)
        _root = _xmlUtils.getRootNode()
        _versionDict = _xmlUtils.parseXml2Dict(_root,None,False)

        if not _versionDict.has_key('updatetime'):
            _versionDict['updatetime'] = ''
        else:
            try:
                _versionDict['updatetime'] = time.strftime("%Y-%m-%d %H:%M:%S",time.localtime(string.atof(_versionDict['updatetime'])))
            except BaseException, e:
                print str(traceback.format_exc())
        
        if os.path.exists(self.autoUpdatePath):        
            _autoUpdate = fileUtils.getFileContent(self.autoUpdatePath)
            _versionDict['autoUpdate'] = _autoUpdate
            
        return _versionDict

    '''file content as follows:
    <?xml version="1.0" encoding="Utf-8"?>
    <root>
    <time>1403644039</time>
    <host id="192.168.5.126" status="false" lasterr="RmFpbGVkOiBUQ1AgY29ubmVjdGlvbiByZXNldCBieSBwZWVyIFstNTk2MV0gKHFwaWQvc3lzL3NzbC9Tc2xTb2NrZXQuY3BwOjIzNik=" />
    </root>
    '''
    def getStatusForLog(self):
        if os.path.isfile(self.logXml)==False:
            return {}
        
        _statusDict = {}
        _xmlUtils = fileUtils.XmlUtils(self.logXml)
        children = _xmlUtils.getAllChildren()
        for childNode in children:
            childTag = childNode.tag
            if  childTag=='time':
                _statusDict['time'] = string.atoi(childNode.text)
            if  childTag=='host':
                _id = childNode.get('id')
                _status = childNode.get('status','')
                _reason = base64.b64decode(childNode.get('lasterr',''))
                _statusDict[_id] = {'status':_status,'reason':_reason}
                
        return _statusDict
    
    '''file content as follows:
    <?xml version="1.0" encoding="Utf-8"?>
    <root>
        <host id="192.168.5.126" time="1403002419"/>
    </root>
    '''
    def getStatusForCmd(self):
        if os.path.isfile(self.cmdXml)==False:
            return {}
        
        _statusDict = {}
        _xmlUtils = fileUtils.XmlUtils(self.cmdXml)
        children = _xmlUtils.getAllChildren('host')
        for childNode in children:
            childText = childNode.text
            _id = childNode.get('id')
            _time = None
            if childNode.get('time').strip()!='':
                _time = string.atoi(childNode.get('time'))
            _statusDict[_id] = {'time':_time}
                
        return _statusDict
    
    def getHostsInUse(self):
        _xmlUtils = fileUtils.XmlUtils(self.clientXml)
        brokerNodes = _xmlUtils.getAllChildren('broker')
        
        _hostDict ={}
        for brokerNode in brokerNodes:
            _id = brokerNode.get('id')
            _host = brokerNode.find('host')
            _isUse = brokerNode.get('inuse')
           
            if ((_host is not None or _host.strip()!='') and _isUse=='yes'):
                _hostDict[_id] = _host.text
                
        return _hostDict
    
    def getNpaiStatus(self,localLang):
         disconnectImg = 'state_disconnect_' + localLang + '.gif'
         connectImg = 'state_connect_' + localLang + '.gif'
         
         cmdStatus = self.getStatusForCmd()
         logStatus = self.getStatusForLog()
         clientInfo = self.getHostsInUse()
         
         logText = ''
         cmdText = ''
         status = {}
         
         #cmdStatusMsg={"success":"{% trans 'NPAI.status.cmd.succ' %}","fail":"{% trans 'NPAI.status.cmd.fail' %}" ,"timeout":"{% trans 'NPAI.status.cmd.timeout' %}"}
         #logStatusMsg={"success":"{% trans 'NPAI.status.log.succ' %}","fail":"{% trans 'NPAI.status.log.fail' %}" ,"timeout":"{% trans 'NPAI.status.log.timeout' %}"}
         
         cmdStatusMsg={"success":_('NPAI.status.cmd.succ'),"fail":_('NPAI.status.cmd.fail') ,"timeout":_('NPAI.status.cmd.timeout')}
         logStatusMsg={"success":_('NPAI.status.log.succ'),"fail":_('NPAI.status.log.fail') ,"timeout":_('NPAI.status.log.timeout')}
         
         if not logStatus.has_key('time'):
             logStatus['time'] = None
         
         if(self.isTimeOut(logStatus['time'])):
             logText = logStatusMsg["timeout"]
             for id,host in clientInfo.items():
                 if cmdStatus.has_key(host):
                     if(self.isTimeOut(cmdStatus[host]['time'])):
                         cmdText = cmdStatusMsg["timeout"]
                     else:
                         cmdText = cmdStatusMsg["success"]
                 else:
                         cmdText = cmdStatusMsg["fail"] + 'the id is not exist in xml'
                 title = cmdText+'<br/>' + logText
                 
                 #status[id] = {'imgName':disconnectImg,'title':title,'class':'tipBox'}
                 _img = '<img src="/medias/app/npai/images/' + disconnectImg + '" width="65" height="23" alt="connect" title="'+ title +'" style="vertical-align:top;" class="tipBox"/>'
                 status[id]= {'img':_img}
         else:
              for id,host in clientInfo.items():
                  if logStatus.has_key(host):
                      if cmdStatus.has_key(host):
                          if(self.isTimeOut(cmdStatus[host]['time'])):
                              cmdText = cmdStatusMsg["timeout"]
                              if(logStatus[host]['status']=='true'):
                                  logText = logStatusMsg["success"]
                              else:
                                  logText = logStatusMsg["fail"] + logStatus[host]['reason']
                          else:
                              cmdText = cmdStatusMsg["success"]
                              if(logStatus[host]['status']=='true'):
                                 logText = True;
                                 cmdText = True;
                              else:
                                 logText = logStatusMsg["fail"] + logStatus[host]['reason']
                      else:
                          cmdText = cmdStatusMsg["fail"] + " the id is not exist";
                          if(logStatus[host]['status']=='true'):
                              logText = logStatusMsg["success"]
                          else:
                              logText = logStatusMsg["fail"] + logStatus[host]['reason']
                   
                  else:
                       if cmdStatus.has_key(host):
                           logText = "NPAI's log interface is error, reason: the id is not exist"
                           if(self.isTimeOut(cmdStatus[host]['time'])):
                               cmdText = cmdStatusMsg["timeout"]
                           else:
                               cmdText = cmdStatusMsg["success"]
                       else:
                           cmdText = cmdStatusMsg["fail"] + " the id is not exist"
                           logText = logStatusMsg["fail"] + " the id is not exist"
                
                  if(logText==True and cmdText==True):
                      status[id] = {'img':'<img src="/medias/app/npai/images/' + connectImg + '" width="65" height="23" alt="connect" style="vertical-align:top;"/>'}
                     
                      #status[id] = {'imgName':connectImg,'title':'','class':''}
                  else:
                      title = cmdText +'<br/>' + logText;
                      _img = '<img src="/medias/app/npai/images/' + disconnectImg + '" width="65" height="23" alt="connect" title="' + title +'" style="vertical-align:top;" class="tipBox"/>'
                      status[id] = {'img':_img}
                      #status[id] = {'imgName':disconnectImg,'title':title,'class':'tipBox'}
         return status            
         
    def isTimeOut(self, detectTime):
        data = fileUtils.getFileContent(self.timeout)
        if data is None or detectTime is None:
            return True
        else:
            interval =  string.atoi(data.strip())
            now = time.time()
            if (now-detectTime)>interval:
                return True
            return False
   
    def recoverSetting(self):
        brokerNodesDict = self.getRecoverXmlInfo()
        
        #print brokerNodesDict
        
        _xmlUtils = fileUtils.XmlUtils(self.clientXml)
        brokerNodes = _xmlUtils.getAllChildren('broker')
        
        def formatValue(key,oldValue,dictValue):
                #host value uses old value
                if key=='host':
                    return None
                else:
                    if dictValue is None:
                        dictValue=''
                    return dictValue
                
                return None
        
        for brokerNode in brokerNodes:
            _brokerId = brokerNode.get('id')
            brokerNodeDict = brokerNodesDict.get(_brokerId)
            if brokerNodeDict is None:
                continue
            _xmlUtils.setLeafNodeValue(brokerNode, brokerNodeDict, '',formatValue)
            # _xmlUtils.setLeafNodeValue(brokerNode, brokerNodeDict, '')
            
        _xmlUtils.writeXml()
        #"setting.recover.succ"
        #"setting.recover.fail"
        return True
    
    def getRecoverXmlInfo(self):
        _xmlUtils = fileUtils.XmlUtils(self.recoverXml)
        brokerNodes = _xmlUtils.getAllChildren('broker')
        
        brokerNodesDict = {}
        
        for brokerNode in brokerNodes:
            _borkerId = brokerNode.get('id')
            '''
            if _borkerId=='0':
                continue
            '''
            nodeDict = _xmlUtils.parseXml2Dict(brokerNode,None,False)
            brokerNodesDict[_borkerId] = nodeDict

        return brokerNodesDict
        
    def autoUpdateSetting(self,value):
        if os.path.exists(self.autoUpdatePath):        
            fileUtils.writeFile(value,self.autoUpdatePath)
            return True
        return False
    
    def updateNpai(self,file, info = {}):
        _targetFilePath = '/tmp'
        _targetFileName = 'updateNpai.dat';
        
        filePath = fileUtils.handleUploadFile(file,_targetFilePath,_targetFileName)

        cmd = "cd /opt/nsfocus/NPAI/bin/update/ && python manu_update.py --file " + filePath
        status = os.system(cmd)>>8

        if status==0:
            return True
        else:
            return False
    
    def getDebugInfo(self):
        cmd = "cd /opt/nsfocus/NPAI/bin/tools && python getinfos.py"
        result = ''
        re = os.popen(cmd).readlines()
        if len(re)>0:
            result = re[0].strip()
        return result
        
if __name__ == '__main__' :
    pass
