# coding: utf-8
import os, sys, re
import time
import shutil
import subprocess
import traceback
from xml.dom.minidom import parse
import lxml.etree
import _winreg
from xml.etree.ElementTree import ElementTree
import simplejson as json
from distutils.version import LooseVersion
import subprocess
import logging


def get_nsc_root():
    if (os.name == 'nt'):
        ROOT_KEY = "Software\\Nsfocus\\Nsc"
        try:
            handle = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, ROOT_KEY)
        except Exception, e:
            return os.environ['NSC_HOME']
            pass
        return _winreg.QueryValueEx(handle, 'path')[0]
    else:
        return os.environ['NSC_HOME']


nsc_dir = get_nsc_root()

source_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'source')
ctme = time.strftime('%Y%m%d%H%M%S')

svc_exe = {'webdaemon': 'webdaemon',
           'toid': 'toid',
           'webtoid': 'webtoid',
           'nscsyslogd': 'syslogd',
           'nscsnmptrapd': 'snmptrapd',
           'nscntunnel': 'ntunnel',
           'nsync': 'nsync',
           'cron': 'crond',
           'nscens': 'nscens',
           'nscftps': 'nscftps',
           'nscqserver': 'nscqserver',
           'nscqpid': 'qpidd',
           'nscache': 'nscache',
           'webtoidgetter': 'webtoidgetter',
           "nscaserver": "nscaserver",
           "nscngpa": 'nscngpa',
           "nscrserver": 'nscrserver'
           }

services = ['webdaemon',
            'nscrserver',
            ]


def spawn(cmd):
    pipe = subprocess.Popen(cmd + " 2>&1", shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
                            stdin=subprocess.PIPE)
    output = pipe.communicate()[0]
    logging.info(cmd)
    logging.info(output)
    res = pipe.returncode
    if res != 0:
        print "Failed to run cmd " + cmd
        print output
        logging.info(res)
        return False
    return True


def stop_service():
    for svc_name in services:
        exe_name = svc_exe[svc_name]
        _stop(svc_name, exe_name)


def _stop(svc_name, exe_name):
    tryTimes = 3
    count = 0
    timeInterval = 1
    while (os.system('net stop ' + svc_name) and count < tryTimes):
        count += 1
        time.sleep(timeInterval * count)
    try:
        os.system('taskkill /im %s.exe /f' % exe_name)
    except BaseException, e:
        pass


def start_service():
    for i in range(-1, -len(services) - 1, -1):
        spawn("sc start %s " % services[i])
        time.sleep(1)
    pass


def copy_fold(src, dst):
    base = os.path.basename(src)
    dst = os.path.join(dst, base)
    names = os.listdir(src)
    if not os.path.exists(dst):
        os.makedirs(dst)
    for name in names:
        srcname = os.path.join(src, name)
        if os.path.isdir(srcname):
            copy_fold(srcname, dst)
        else:
            try:
                old_name = os.path.join(dst, name)
                if os.path.exists(old_name):
                    bak_name = old_name + '.' + ctme
                    shutil.copy2(old_name, bak_name)
                shutil.copy2(srcname, dst)
            except Exception, e:
                print e


if __name__ == '__main__':
    try:
        print "start updating"
        #print "stop service"
        #stop_service()

        names = os.listdir(source_dir)
        for name in names:
            srcname = os.path.join(source_dir, name)
            if os.path.isdir(srcname):
                copy_fold(srcname, nsc_dir)
            else:
                old_name = os.path.join(nsc_dir, name)
                if os.path.exists(old_name):
                    bak_name = old_name + '.' + ctme
                    shutil.copy2(old_name, bak_name)
                shutil.copy2(srcname, dst)

        #print 'start service'
        #start_service()
        print 'Update is completed'
    except Exception, e:
        print e
        print 'update failed'
